# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013 RFUS TEAM <gau1991@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 2, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

#some basic import
import sys, os

#confing parser to read configuration file
import ConfigParser

#for wait function
import time

#for subprocess commands
import subprocess

#for notify
from gi.repository import Dee
# FIXME: Some weird bug in Dee or PyGI makes Dee fail unless we probe
#        it *before* we import the Unity module... ?!
_m = dir(Dee.SequenceModel)
from gi.repository import Notify

#for unity launcher
from gi.repository import Unity

#for pathname matching in specified pattern
import glob

#for naming temp file
import tempfile
 
#for translation support
import gettext
from gettext import gettext as _
gettext.textdomain('rfus')

#for GTK3
from gi.repository import Gtk # pylint: disable=E0611
import logging
logger = logging.getLogger('rfus')

#from some nearest python files
from rfus_lib import Window
from rfus.AboutRfusDialog import AboutRfusDialog
from rfus.PreferencesRfusDialog import PreferencesRfusDialog


# See rfus_lib.Window.py for more details about how this class work

#to read config section map
def ConfigSectionMap(section,Config):
    dict1 = {}
    options = Config.options(section)
    for option in options:
        try:
            dict1[option] = Config.get(section, option)
            if dict1[option] == -1:
                DebugPrint("skip: %s" % option)
        except:
            print("exception on %s!" % option)
            dict1[option] = None
    return dict1


#rfus windows class
class RfusWindow(Window):
    __gtype_name__ = "RfusWindow"

    #for next1 button clicked
    def on_next1_clicked(self,button):
        if self.disknm == None:
            dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR,Gtk.ButtonsType.CANCEL, "Please Select Partition")
            dialog.run()
            dialog.destroy()
        else:
            self.nb1.set_current_page(1)

    #for quit1 button clicked
    def on_quit1_clicked(self,button):
        sys.exit(0)
    
    #for quit2 button clicked
    def on_quit2_clicked(self,button):
        sys.exit(0)

    #for back1 button clicked
    def on_back1_clicked(self,button):
        self.nb1.set_current_page(0)

    #to get radio button toggled in tree view
    def on_cell_radio_toggled(self, widget, path):
        self.disknm = self.disk[int(path)]
        selected_path = Gtk.TreePath(path)
        for row in self.liststore:
            row[2] = (row.path == selected_path)

    #to define new file format
    def on_ownformat_toggled(self, button):
        if button.get_active():
            self.format.append('new')
            self.ui.fname.set_visible(True)
            self.ui.lfname.set_visible(True)
            self.ui.lfsize.set_visible(True)
            self.ui.fsize.set_visible(True)
            self.ui.lfheader.set_visible(True)
            self.ui.fheader.set_visible(True)
            self.ui.lffooter.set_visible(True)
            self.ui.ffooter.set_visible(True)
        else:
            self.format.remove('new')
            self.ui.fname.set_visible(False)
            self.ui.lfname.set_visible(False)
            self.ui.lfsize.set_visible(False)
            self.ui.fsize.set_visible(False)
            self.ui.lfheader.set_visible(False)
            self.ui.fheader.set_visible(False)
            self.ui.lffooter.set_visible(False)
            self.ui.ffooter.set_visible(False)

    #Here Onwards all are the format selection the check buttons, add selected checkbox value in self.format 
    def on_pdf_toggled(self, button):
        if button.get_active():
            self.format.append('pdf')
        else:
            self.format.remove('pdf')
            self.ui.all.set_active(False)     
       

    def on_art_toggled(self, button):
        if button.get_active():
            self.format.append('art')
        else:
            self.format.remove('art')
            self.ui.all.set_active(False)   
        

    def on_gif_toggled(self, button):
        if button.get_active():
            self.format.append('gif')
        else:
            self.format.remove('gif')
            self.ui.all.set_active(False)   
        

    def on_jpg_toggled(self, button):
        if button.get_active():
            self.format.append('jpg')
        else:
            self.format.remove('jpg')
            self.ui.all.set_active(False)   
        

    def on_png_toggled(self, button):
        if button.get_active():
            self.format.append('png')
        else:
            self.format.remove('png')
            self.ui.all.set_active(False)   
        

    def on_bmp_toggled(self, button):
        if button.get_active():
            self.format.append('bmp')
        else:
            self.format.remove('bmp')
            self.ui.all.set_active(False)   
       

    def on_tif_toggled(self, button):
        if button.get_active():
            self.format.append('tif')
        else:
            self.format.remove('tif')
            self.ui.all.set_active(False)   
       

    def on_avi_toggled(self, button):
        if button.get_active():
            self.format.append('avi')
        else:
            self.format.remove('avi')
            self.ui.all.set_active(False)   
        

    def on_mpg_toggled(self, button):
        if button.get_active():
            self.format.append('mpg')
        else:
            self.format.remove('mpg')
            self.ui.all.set_active(False)   
       

    def on_swf_toggled(self, button):
        if button.get_active():
            self.format.append('swf')
        else:
            self.format.remove('swf')
            self.ui.all.set_active(False)   

    def on_mov_toggled(self, button):
        if button.get_active():
            self.format.append('mov')
        else:
            self.format.remove('mov')
            self.ui.all.set_active(False)   

    def on_odt_toggled(self, button):
        if button.get_active():
            self.format.append('odt')
        else:
            self.format.remove('odt')
            self.ui.all.set_active(False)   

    def on_ods_toggled(self, button):
        if button.get_active():
            self.format.append('ods')
        else:
            self.format.remove('ods')
            self.ui.all.set_active(False)   

    def on_odp_toggled(self, button):
        if button.get_active():
            self.format.append('odp')
        else:
            self.format.remove('odp')
            self.ui.all.set_active(False)   

    def on_odg_toggled(self, button):
        if button.get_active():
            self.format.append('odg')
        else:
            self.format.remove('odg')
            self.ui.all.set_active(False)   

    def on_odc_toggled(self, button):
        if button.get_active():
            self.format.append('odc')
        else:
            self.format.remove('odc')
            self.ui.all.set_active(False)   

    def on_odf_toggled(self, button):
        if button.get_active():
            self.format.append('odf')
        else:
            self.format.remove('odf')
            self.ui.all.set_active(False)   

    def on_odi_toggled(self, button):
        if button.get_active():
            self.format.append('odi')
        else:
            self.format.remove('odi')
            self.ui.all.set_active(False)   

    def on_odm_toggled(self, button):
        if button.get_active():
            self.format.append('odm')
        else:
            self.format.remove('odm')
            self.ui.all.set_active(False)   

    def on_doc_toggled(self, button):
        if button.get_active():
            self.format.append('doc')
        else:
            self.format.remove('doc')
            self.ui.all.set_active(False)   

    def on_pst_toggled(self, button):
        if button.get_active():
            self.format.append('pst')
        else:
            self.format.remove('pst')
            self.ui.all.set_active(False)   


    def on_ost_toggled(self, button):
        if button.get_active():
            self.format.append('ost')
        else:
            self.format.remove('ost')
            self.ui.all.set_active(False)   

    def on_dbx_toggled(self, button):
        if button.get_active():
            self.format.append('dbx')
        else:
            self.format.remove('dbx')
            self.ui.all.set_active(False)   

    def on_idx_toggled(self, button):
        if button.get_active():
            self.format.append('idx')
        else:
            self.format.remove('idx')
            self.ui.all.set_active(False)   

    def on_mbx_toggled(self, button):
        if button.get_active():
            self.format.append('mbx')
        else:
            self.format.remove('mbx')
            self.ui.all.set_active(False)   

    def on_wpc_toggled(self, button):
        if button.get_active():
            self.format.append('wpc')
        else:
            self.format.remove('wpc')
            self.ui.all.set_active(False)   

    def on_htm_toggled(self, button):
        if button.get_active():
            self.format.append('htm')
        else:
            self.format.remove('htm')
            self.ui.all.set_active(False)   

    def on_pdf_toggled(self, button):
        if button.get_active():
            self.format.append('pdf')
        else:
            self.format.remove('pdf')
            self.ui.all.set_active(False)   

    def on_mail_toggled(self, button):
        if button.get_active():
            self.format.append('mail')
        else:
            self.format.remove('mail')
            self.ui.all.set_active(False)   


    def on_pgp_toggled(self, button):
        if button.get_active():
            self.format.append('pgp')
        else:
            self.format.remove('pgp')
            self.ui.all.set_active(False)   

    def on_pdg_toggled(self, button):
        if button.get_active():
            self.format.append('pdg')
        else:
            self.format.remove('pdg')
            self.ui.all.set_active(False)   

    def on_rpm_toggled(self, button):
        if button.get_active():
            self.format.append('rpm')
        else:
            self.format.remove('rpm')
            self.ui.all.set_active(False)   

    def on_wav_toggled(self, button):
        if button.get_active():
            self.format.append('wav')
        else:
            self.format.remove('wav')
            self.ui.all.set_active(False)   

    def on_ra_toggled(self, button):
        if button.get_active():
            self.format.append('ra')
        else:
            self.format.remove('ra')
            self.ui.all.set_active(False)   

    def on_dat_toggled(self, button):
        if button.get_active():
            self.format.append('dat')
        else:
            self.format.remove('dat')
            self.ui.all.set_active(False)   

    def on_zip_toggled(self, button):
        if button.get_active():
            self.format.append('zip')
        else:
            self.format.remove('zip')
            self.ui.all.set_active(False)   

    def on_java_toggled(self, button):
        if button.get_active():
            self.format.append('java')
        else:
            self.format.remove('java')
            self.ui.all.set_active(False)   

    def on_msf_toggled(self, button):
        if button.get_active():
            self.format.append('msf')
        else:
            self.format.remove('msf')
            self.ui.all.set_active(False)   

    def on_max_toggled(self, button):
        if button.get_active():
            self.format.append('max')
        else:
            self.format.remove('max')
            self.ui.all.set_active(False)   

    def on_pins_toggled(self, button):
        if button.get_active():
            self.format.append('pins')
        else:
            self.format.remove('pins')
            self.ui.all.set_active(False)   

    #if all format selection button toggled
    def on_all_toggled(self, button):
        if button.get_active():
            self.ui.art.set_active(True)            
            self.ui.gif.set_active(True)
            self.ui.jpg.set_active(True)
            self.ui.png.set_active(True)
            self.ui.bmp.set_active(True)
            self.ui.tif.set_active(True)
            self.ui.avi.set_active(True)
            self.ui.mpg.set_active(True)
            self.ui.swf.set_active(True)
            self.ui.mov.set_active(True)
            self.ui.odt.set_active(True)
            self.ui.ods.set_active(True)
            self.ui.odp.set_active(True)
            self.ui.odg.set_active(True)
            self.ui.odc.set_active(True)
            self.ui.odf.set_active(True)
            self.ui.odi.set_active(True)
            self.ui.odm.set_active(True)
            self.ui.doc.set_active(True)
            self.ui.pst.set_active(True)
            self.ui.ost.set_active(True)            
            self.ui.dbx.set_active(True)
            self.ui.idx.set_active(True)
            self.ui.mbx.set_active(True)
            self.ui.wpc.set_active(True)
            self.ui.htm.set_active(True)
            self.ui.pdf.set_active(True)
            self.ui.mail.set_active(True)
            self.ui.pgp.set_active(True)
            self.ui.pdg.set_active(True)
            self.ui.rpm.set_active(True)
            self.ui.wav.set_active(True)
            self.ui.ra.set_active(True)
            self.ui.dat.set_active(True)
            self.ui.zip.set_active(True)
            self.ui.java.set_active(True)
            self.ui.msf.set_active(True)
            self.ui.max.set_active(True)            
            self.ui.pins.set_active(True)

    #for empty folder browsing button
    def on_folder_choose_clicked(self,button):
        dialog = Gtk.FileChooserDialog("Please choose a folder", self,
            Gtk.FileChooserAction.SELECT_FOLDER,
            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
             "Select", Gtk.ResponseType.OK))
        dialog.set_default_size(800, 400)
        response = dialog.run()
        if response == Gtk.ResponseType.OK:
           self.ui.folder_selection.set_text(dialog.get_filename())
        dialog.destroy()

    #on next2 button clicked
    def on_next2_clicked(self,button):
        if len(self.format) == 0 or self.ui.folder_selection.get_text() == "":
            dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR,Gtk.ButtonsType.CANCEL, "Please Select All Fields")
            dialog.run()
            dialog.destroy()

        elif self.ui.ownformat.get_active() and (self.ui.fname.get_text() == "" or self.ui.fsize.get_text() == "" or self.ui.fheader.get_text() == ""):
            dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR,Gtk.ButtonsType.CANCEL, "Please Select All Fields")
            dialog.run()
            dialog.destroy()
            
        elif self.ui.ownformat.get_active() and (not self.ui.fsize.get_text().isdigit()):
            dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR,Gtk.ButtonsType.CANCEL, "Please Specify Valid Size")
            dialog.run()
            dialog.destroy()


        elif len(os.listdir(self.ui.folder_selection.get_text())) > 0:
            dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR,Gtk.ButtonsType.CANCEL, "Please Select Empty Folder")
            dialog.run()
            dialog.destroy()
        

        else:
            size = 0
            if os.path.isfile(self.cfpath):
                Config = ConfigParser.ConfigParser()
                Config.read(self.cfpath)
                size = ConfigSectionMap("rfus", Config)['size']
                size = int(size)
                         
            self.ui.disk_label.set_text("Step 1/2 = Scanning disk")
            self.nb1.set_current_page(2)
            if size > 0:
                self.write_to_file_new(size)
            else:
                self.write_to_file_def()

            self.ui.cancel.show()
            self.ui.quit2.hide()
            self.recover_start()
            
    #actual recovery process starting in with scalpel
    def recover_start(self):
        clo = ""
        if os.path.isfile(self.cfpath):
            Config = ConfigParser.ConfigParser()
            Config.read(self.cfpath)
            try:
                clo = ConfigSectionMap("rfus", Config)['clo']        
            except:
                clo = ""
        cmd = 'pkexec scalpel '+self.disknm+' -o "'+self.ui.folder_selection.get_text()+'" -c ' + self.fnm + ' ' + clo
        self.proc = subprocess.Popen(cmd,shell=True, stdout=subprocess.PIPE)
        myl = []
        fper=0.0
        self.launcher.set_property("progress_visible", True)
        while True:
	        line = self.proc.stdout.read(1)
	        if not line:
		        break
	        myl.append(line)
	        cnt = len(myl)
	        if line == '%':
	        	mper = myl[cnt-6]+myl[cnt-5]+myl[cnt-4]+myl[cnt-3]+myl[cnt-2]
		        nper = mper.rstrip()
		        fper = float(nper)
                fper = fper/100.0
		        self.ui.progressbar1.set_fraction(fper)
                self.launcher.set_property("progress",fper)
                if fper == 1.0:
                    self.ui.disk_label.set_text("Step 2/2 = Carving disk")
		        myl = []
	        elif line == 'A' and myl[cnt-2] == 'T' and myl[cnt-3] == 'E':
		        rtime = myl[cnt-12]+myl[cnt-11]+myl[cnt-10]+myl[cnt-9]+myl[cnt-8]+myl[cnt-7]+myl[cnt-6]+myl[cnt-5]+myl[cnt-4]
		        ntime = rtime.rstrip()
                self.ui.label9.set_text(ntime)
		        myl = [] 
            
            #this removes freezing of window
	        while Gtk.events_pending():
                 Gtk.main_iteration()

        streamdata = self.proc.communicate()[0]
        rc = self.proc.returncode
        self.launcher.set_property("progress_visible", False)
        if rc == 0:
            self.ui.disk_label.set_text("Recovering done, No Errors Found")
            self.current_notification = Notify.Notification.new("Rfus","Recovering Done, No Errors Found","/usr/share/rfus/media/rfus6.png")
	        self.current_notification.show()
            os.system('nautilus "'+self.ui.folder_selection.get_text()+ '"')
        else:
            self.current_notification = Notify.Notification.new("Rfus","Error in Executing Scalpel","/usr/share/rfus/media/rfus6.png")
	        self.current_notification.show()
            self.ui.disk_label.set_text("Error in Executing Scalpel")
        self.ui.cancel.hide()
        self.ui.quit2.show()
        os.system('rm ' + self.fnm)
            
    #it creates tmp config file to store selected format with size given by user
    def write_to_file_new(self,size):

        t = tempfile.NamedTemporaryFile(dir="/tmp")
        self.fnm = t.name + '-rfus.conf'
        f = open(self.fnm , 'w')

        #code to write tmp config file
        if 'new' in self.format:
            f.write("       "+self.ui.fname.get_text()+"    y   "+self.ui.fsize.get_text()+"    "+self.ui.fheader.get_text()+"  "+self.ui.ffooter.get_text()+"\n")
        
        if 'art' in self.format:
            f.write("       art y	"+size+"	\\x4a\\x47\\x04\\x0e	\\xcf\\xc7\\xcb\n")
            f.write("       art y 	"+size+"	\\x4a\\x47\\x03\\x0e	\\xd0\\xcb\\x00\\x00\n")
        if 'gif' in self.format:
            f.write("       gif y	"+size+"		\\x47\\x49\\x46\\x38\\x37\\x61	\\x00\\x3b\n")
            f.write("       gif y 	"+size+"		\\x47\\x49\\x46\\x38\\x39\\x61	\\x00\\x3b\n")
        if 'jpg' in self.format:
            f.write("       jpg y	"+size+"	\\xff\\xd8\\xff\\xe0\\x00\\x10	\\xff\\xd9\n")
        if 'png' in self.format:
            f.write("       png y	"+size+"	\\x50\\x4e\\x47?	\\xff\\xfc\\xfd\\xfe\n")
        if 'bmp' in self.format:
            f.write("       bmp y	"+size+"	BM??\\x00\\x00\\x00\n")
        if 'tif' in self.format:
            f.write("       tif y	"+size+"	\\x49\\x49\\x2a\\x00\n")
            f.write("       tif y	"+size+"	\\x4D\\x4D\\x00\\x2A\n")
        if 'avi' in self.format:
            f.write("       avi y	"+size+" RIFF????AVI\n")
        if 'mov' in self.format:
            f.write("       mov y	"+size+"	????moov\n")
            f.write("       mov y	"+size+"	????mdat\n")
            f.write("       mov y	"+size+"	????widev\n")
            f.write("       mov y	"+size+"	????skip\n")
            f.write("       mov y	"+size+"	????free\n")
            f.write("       mov y	"+size+"	????idsc\n")
            f.write("       mov y	"+size+"	????pckg\n")

        if 'mpg' in self.format:
            f.write("       mpg y	"+size+"	\\x00\\x00\\x01\\xba	\\x00\\x00\\x01\\xb9\n")
            f.write("       mpg y 	"+size+"	\\x00\\x00\\x01\\xb3	\\x00\\x00\\x01\\xb7\n")
        if 'swf' in self.format:
            f.write("       fws y	"+size+"	FWS\n")
        if 'doc' in self.format:
            f.write("       doc y	"+size+"  \\xd0\\xcf\\x11\\xe0\\xa1\\xb1\\x1a\\xe1\\x00\\x00 \\xd0\\xcf\\x11\\xe0\\xa1\\xb1\\x1a\\xe1\\x00\\x00 NEXT\n")
            f.write("       doc y	"+size+"  \\xd0\\xcf\\x11\\xe0\\xa1\\xb1\n")
        if 'pst' in self.format:
            f.write("       pst y	"+size+"	\\x21\\x42\\x4e\\xa5\\x6f\\xb5\\xa6\n")

        if 'ost' in self.format:
            f.write("       ost y	"+size+" 	\\x21\\x42\\x44\\x4e\n")
            
        if 'dbx' in self.format:
            f.write("       dbx y	"+size+"	\\xcf\\xad\\x12\\xfe\\xc5\\xfd\\x74\\x6f\n")

        if 'idx' in self.format:
            f.write("       idx y	"+size+"	\\x4a\\x4d\\x46\\x39\n")
            
        if 'mbx' in self.format:
            f.write("       mbx y	"+size+"	\\x4a\\x4d\\x46\\x36\n")

        if 'wpc' in self.format:
            f.write("       wpc y	"+size+"	?WPC\n")

        if 'htm' in self.format:
            f.write("       htm n	"+size+"   <html			</html>\n")
            
        if 'pdf' in self.format:
            f.write("       pdf y	"+size+"	%PDF  %EOF\\x0d	REVERSE\n")
            f.write("       pdf y	"+size+"	%PDF  %EOF\\x0a	REVERSE\n")
        if 'mail' in self.format:
            f.write("       mail    y	"+size+"	 \\x41\\x4f\\x4c\\x56\\x4d\n")
            
        if 'pgd' in self.format:
            f.write("       pgd y	"+size+"	\\x50\\x47\\x50\\x64\\x4d\\x41\\x49\\x4e\\x60\\x01\n")
            
        if 'pgp' in self.format:
            f.write("       pgp y	"+size+"	\\x99\\x00\n")
            f.write("       pgp y	"+size+"	\\x95\\x01\n")
            f.write("       pgp y	"+size+"	\\x95\\x00\n")
            f.write("       pgp y	"+size+"	\\xa6\\x00\n")
        if 'rpm' in self.format:
            f.write("       rpm y	"+size+"	\\xed\\xab\n")
            
        if 'wav' in self.format:
            f.write("       wav y	"+size+"	RIFF????WAVE\n")

        if 'ra' in self.format:
            f.write("       ra  y	"+size+"	\\x2e\\x72\\x61\\xfd\n")
            f.write("       ra  y	"+size+"	.RMF\n")
        if 'dat' in self.format:
            f.write("       dat y	"+size+"	regf\n")
            f.write("       dat y	"+size+"	CREG\n")
        if 'zip' in self.format:
            f.write("       zip y	"+size+"	PK\\x03\\x04	\\x3c\\xac\n")
           
        if 'java' in self.format:
            f.write("       java    y	"+size+"	\\xca\\xfe\\xba\\xb\n")

        if 'max' in self.format:
            f.write("       max y     "+size+"    \\x56\\x69\\x47\\x46\\x6b\\x1a\\x00\\x00\\x00\\x00   \\x00\\x00\\x05\\x80\\x00\\x00\n")

        if 'pins' in self.format:
            f.write("       pins    y     "+size+"     \\x50\\x49\\x4e\\x53\\x20\\x34\\x2e\\x32\\x30\\x0d\n")
            
        if 'odt' in self.format:
            f.write("       odt y   "+size+"    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.textPK META-INF/manifest.xmlPK????????????????????\n")
            
        if 'ods' in self.format:
            f.write("       ods y   "+size+"    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.spreadsheetPK  META-INF/manifest.xmlPK????????????????????\n")

        if 'odp' in self.format:
            f.write("       odp y   "+size+"    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.presentationPK META-INF/manifest.xmlPK????????????????????\n")

        if 'odg' in self.format:
            f.write("       odg y   "+size+"    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.graphicsPK META-INF/manifest.xmlPK????????????????????\n")
            
        if 'odc' in self.format:
            f.write("       odc y   "+size+"    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.chartPK    META-INF/manifest.xmlPK????????????????????\n")
           
        if 'odf' in self.format:
            f.write("       odf y   "+size+"    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.formulaPK  META-INF/manifest.xmlPK????????????????????")

        if 'odi' in self.format:
            f.write("       odi y   "+size+"    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.imagePK    META-INF/manifest.xmlPK????????????????????\n")

        if 'odm' in self.format:
            f.write("       odm y   "+size+"    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.text-masterPK  META-INF/manifest.xmlPK????????????????????\n")

        if 'swx' in self.format:
            f.write("       sxw y   "+size+"    PK????????????????????????????mimetypeapplication/vnd.sun.xml.writerPK  META-INF/manifest.xmlPK????????????????????\n")
        
        if 'msf' in self.format:
            f.write('       msf y   '+size+'    //\s<!--\s<mdb:mork:z\sv="1.4"/>\s-->?<\s<(a=c)>\s//\s(f=iso-8859-1)    //\s<!--\s<mdb:mork:z\sv="1.4"/>\s-->?<\s<(a=c)>\s//\s(f=iso-8859-1)    NEXT\n')

        f.close()

    #it creates tmp config file to store selected format with default size
    def write_to_file_def(self):
        t = tempfile.NamedTemporaryFile(dir="/tmp")
        self.fnm = t.name + '-rfus.conf'
        f = open(self.fnm , 'w')
        #to write user defined format in tmp config file
        if 'new' in self.format:
            f.write("       "+self.ui.fname.get_text()+"    y   "+self.ui.fsize.get_text()+"    "+self.ui.fheader.get_text()+"  "+self.ui.ffooter.get_text()+"\n")      
        
                
        if 'art' in self.format:
            f.write("       art y	150000	\\x4a\\x47\\x04\\x0e	\\xcf\\xc7\\xcb\n")
            f.write("       art y 	150000	\\x4a\\x47\\x03\\x0e	\\xd0\\xcb\\x00\\x00\n")
        if 'gif' in self.format:
            f.write("       gif y	5000000		\\x47\\x49\\x46\\x38\\x37\\x61	\\x00\\x3b\n")
            f.write("       gif y 	5000000		\\x47\\x49\\x46\\x38\\x39\\x61	\\x00\\x3b\n")
        if 'jpg' in self.format:
            f.write("       jpg y	200000000	\\xff\\xd8\\xff\\xe0\\x00\\x10	\\xff\\xd9\n")
        if 'png' in self.format:
            f.write("       png y	20000000	\\x50\\x4e\\x47?	\\xff\\xfc\\xfd\\xfe\n")
        if 'bmp' in self.format:
            f.write("       bmp y	100000	BM??\\x00\\x00\\x00\n")
        if 'tif' in self.format:
            f.write("       tif y	200000000	\\x49\\x49\\x2a\\x00\n")
            f.write("       tif y	200000000	\\x4D\\x4D\\x00\\x2A\n")
        if 'avi' in self.format:
            f.write("       avi y	50000000 RIFF????AVI\n")
        if 'mov' in self.format:
            f.write("       mov y	10000000	????moov\n")
            f.write("       mov y	10000000	????mdat\n")
            f.write("       mov y	10000000	????widev\n")
            f.write("       mov y	10000000	????skip\n")
            f.write("       mov y	10000000	????free\n")
            f.write("       mov y	10000000	????idsc\n")
            f.write("       mov y	10000000	????pckg\n")

        if 'mpg' in self.format:
            f.write("       mpg y	50000000	\\x00\\x00\\x01\\xba	\\x00\\x00\\x01\\xb9\n")
            f.write("       mpg y 	50000000	\\x00\\x00\\x01\\xb3	\\x00\\x00\\x01\\xb7\n")
        if 'swf' in self.format:
            f.write("       fws y	4000000	FWS\n")
        if 'doc' in self.format:
            f.write("       doc y	10000000  \\xd0\\xcf\\x11\\xe0\\xa1\\xb1\\x1a\\xe1\\x00\\x00 \\xd0\\xcf\\x11\\xe0\\xa1\\xb1\\x1a\\xe1\\x00\\x00 NEXT\n")
            f.write("       doc y	10000000  \\xd0\\xcf\\x11\\xe0\\xa1\\xb1\n")
        if 'pst' in self.format:
            f.write("       pst y	500000000	\\x21\\x42\\x4e\\xa5\\x6f\\xb5\\xa6\n")

        if 'ost' in self.format:
            f.write("       ost y	500000000 	\\x21\\x42\\x44\\x4e\n")
            
        if 'dbx' in self.format:
            f.write("       dbx y	10000000	\\xcf\\xad\\x12\\xfe\\xc5\\xfd\\x74\\x6f\n")

        if 'idx' in self.format:
            f.write("       idx y	10000000	\\x4a\\x4d\\x46\\x39\n")
            
        if 'mbx' in self.format:
            f.write("       mbx y	10000000	\\x4a\\x4d\\x46\\x36\n")

        if 'wpc' in self.format:
            f.write("       wpc y	1000000	?WPC\n")

        if 'htm' in self.format:
            f.write("       htm n	50000   <html			</html>\n")
            
        if 'pdf' in self.format:
            f.write("       pdf y	5000000	%PDF  %EOF\\x0d	REVERSE\n")
            f.write("       pdf y	5000000	%PDF  %EOF\\x0a	REVERSE\n")
        if 'mail' in self.format:
            f.write("       mail    y	500000	 \\x41\\x4f\\x4c\\x56\\x4d\n")
            
        if 'pgd' in self.format:
            f.write("       pgd y	500000	\\x50\\x47\\x50\\x64\\x4d\\x41\\x49\\x4e\\x60\\x01\n")
            
        if 'pgp' in self.format:
            f.write("       pgp y	100000	\\x99\\x00\n")
            f.write("       pgp y	100000	\\x95\\x01\n")
            f.write("       pgp y	100000	\\x95\\x00\n")
            f.write("       pgp y	100000	\\xa6\\x00\n")
        if 'rpm' in self.format:
            f.write("       rpm y	1000000	\\xed\\xab\n")
            
        if 'wav' in self.format:
            f.write("       wav y	200000	RIFF????WAVE\n")

        if 'ra' in self.format:
            f.write("       ra  y	1000000	\\x2e\\x72\\x61\\xfd\n")
            f.write("       ra  y	1000000	.RMF\n")
        if 'dat' in self.format:
            f.write("       dat y	4000000	regf\n")
            f.write("       dat y	4000000	CREG\n")
        if 'zip' in self.format:
            f.write("       zip y	10000000	PK\\x03\\x04	\\x3c\\xac\n")
           
        if 'java' in self.format:
            f.write("       java    y	1000000	\\xca\\xfe\\xba\\xb\n")

        if 'max' in self.format:
            f.write("       max y     1000000    \\x56\\x69\\x47\\x46\\x6b\\x1a\\x00\\x00\\x00\\x00   \\x00\\x00\\x05\\x80\\x00\\x00\n")

        if 'pins' in self.format:
            f.write("       pins    y     8000     \\x50\\x49\\x4e\\x53\\x20\\x34\\x2e\\x32\\x30\\x0d\n")
            
        if 'odt' in self.format:
            f.write("       odt y   20000000    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.textPK META-INF/manifest.xmlPK????????????????????\n")
            
        if 'ods' in self.format:
            f.write("       ods y   10000000    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.spreadsheetPK  META-INF/manifest.xmlPK????????????????????\n")

        if 'odp' in self.format:
            f.write("       odp y   10000000    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.presentationPK META-INF/manifest.xmlPK????????????????????\n")

        if 'odg' in self.format:
            f.write("       odg y   10000000    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.graphicsPK META-INF/manifest.xmlPK????????????????????\n")
            
        if 'odc' in self.format:
            f.write("       odc y   10000000    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.chartPK    META-INF/manifest.xmlPK????????????????????\n")
           
        if 'odf' in self.format:
            f.write("       odf y   10000000    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.formulaPK  META-INF/manifest.xmlPK????????????????????")

        if 'odi' in self.format:
            f.write("       odi y   10000000    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.imagePK    META-INF/manifest.xmlPK????????????????????\n")

        if 'odm' in self.format:
            f.write("       odm y   10000000    PK????????????????????????????mimetypeapplication/vnd.oasis.opendocument.text-masterPK  META-INF/manifest.xmlPK????????????????????\n")

        if 'swx' in self.format:
            f.write("       sxw y   10000000    PK????????????????????????????mimetypeapplication/vnd.sun.xml.writerPK  META-INF/manifest.xmlPK????????????????????\n")
        
        if 'msf' in self.format:
            f.write('       msf y   10000000    //\s<!--\s<mdb:mork:z\sv="1.4"/>\s-->?<\s<(a=c)>\s//\s(f=iso-8859-1)    //\s<!--\s<mdb:mork:z\sv="1.4"/>\s-->?<\s<(a=c)>\s//\s(f=iso-8859-1)    NEXT\n')

        f.close()

    #while processing if cancel button clicked
    def on_cancel_clicked(self,button):
        os.system("pkexec killall scalpel")

    #initialization window
    def finish_initializing(self, builder):# pylint: disable=E1002
        """Set up the main window"""
        super(RfusWindow, self).finish_initializing(builder)

        self.AboutDialog = AboutRfusDialog
        self.PreferencesDialog = PreferencesRfusDialog

        # Code for other initialization actions should be added here.        
        Notify.init('Rfus')

        #variable to name tmp file        
        self.fnm = None

        #to tell unity about .desktop file
        self.launcher = Unity.LauncherEntry.get_for_desktop_id("extras-rfus.desktop")

        #to store basic configuration file
        self.hm = os.getenv("HOME")
        self.cfpath = os.path.join(self.hm,'.config/rfus.conf')        

        self.ui.fname.set_visible(False)
        self.ui.lfname.set_visible(False)
        self.ui.lfsize.set_visible(False)
        self.ui.fsize.set_visible(False)
        self.ui.lfheader.set_visible(False)
        self.ui.fheader.set_visible(False)
        self.ui.lffooter.set_visible(False)
        self.ui.ffooter.set_visible(False)

        self.disknm = None
        self.format = []
        self.ui.folder_selection.set_editable(False)        
        self.ui.cancel.hide()
        self.nb1 = self.ui.notebook1
        self.nb1.set_property('show-tabs', False)
        self.nb1.set_current_page(0)
        viewport=builder.get_object("viewport2")

        #to find partions
        self.disk = glob.glob('/dev/sd??')   
        if len(self.disk) == 0:
            self.disk = glob.glob('/dev/hd??')
        self.disk.reverse()
        diskcnt = len(self.disk)
        self.liststore = Gtk.ListStore(str, str, bool)
        i = 0
        while i < diskcnt:
            self.liststore.append([self.disk[i], Gtk.STOCK_DIRECTORY, False])
            i=i+1

        #to define partiton tree view
        treeview = Gtk.TreeView(model=self.liststore)
        renderer_text = Gtk.CellRendererText()
        column_text = Gtk.TreeViewColumn("Partition Name", renderer_text, text=0)
        treeview.append_column(column_text)

        renderer_pixbuf = Gtk.CellRendererPixbuf()        
        column_pixbuf = Gtk.TreeViewColumn("Icon", renderer_pixbuf, stock_id=1)
        treeview.append_column(column_pixbuf)
        
        renderer_radio = Gtk.CellRendererToggle()
        renderer_radio.set_radio(True)
        renderer_radio.connect("toggled", self.on_cell_radio_toggled)
        column_radio = Gtk.TreeViewColumn("Select Your Partition", renderer_radio, active=2)
        treeview.append_column(column_radio)
        
        treeview.show()
        viewport.add(treeview)

        self.disknm = None
